/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */

const { Clutter, Gio, GLib, GObject, Shell, St } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Layout = imports.ui.layout;
const Main = imports.ui.main;

const ClassificationBanner = GObject.registerClass(
class ClassificationBanner extends Clutter.Actor {
    _init(index) {
        const constraint = new Layout.MonitorConstraint({index});
        super._init({
            layout_manager: new Clutter.BinLayout(),
            constraints: constraint,
        });
        this._monitorConstraint = constraint;

        Shell.util_set_hidden_from_pick(this, true);

        this._settings = ExtensionUtils.getSettings();
        this.connect('destroy', () => {
            if (this._fullscreenChangedId)
                global.display.disconnect(this._fullscreenChangedId);
            delete this._fullscreenChangedId;

            this._settings?.run_dispose();
            this._settings = null;
        });

        this._topBanner = new St.BoxLayout({
            style_class: 'classification-banner',
            x_expand: true,
            y_expand: true,
            y_align: Clutter.ActorAlign.START,
        });
        this.add_child(this._topBanner);
        this._settings.bind('top-banner',
            this._topBanner, 'visible',
            Gio.SettingsBindFlags.GET);

        this._bottomBanner = new St.BoxLayout({
            style_class: 'classification-banner',
            x_expand: true,
            y_expand: true,
            y_align: Clutter.ActorAlign.END,
        });
        this.add_child(this._bottomBanner);
        this._settings.bind('bottom-banner',
            this._bottomBanner, 'visible',
            Gio.SettingsBindFlags.GET);

        for (const banner of [this._topBanner, this._bottomBanner]) {
            const label = new St.Label({
                style_class: 'classification-message',
                x_align: Clutter.ActorAlign.CENTER,
                x_expand: true,
            });
            banner.add_child(label);

            this._settings.bind('message',
                label, 'text',
                Gio.SettingsBindFlags.GET);
        }

        const hostLabel = new St.Label({
            style_class: 'classification-system-info',
            text: GLib.get_host_name(),
        });
        this._topBanner.insert_child_at_index(hostLabel, 0);
        this._settings.bind('system-info',
            hostLabel, 'visible',
            Gio.SettingsBindFlags.GET);

        const userLabel = new St.Label({
            style_class: 'classification-system-info',
            text: GLib.get_user_name(),
        });
        this._topBanner.add_child(userLabel);
        this._settings.bind('system-info',
            userLabel, 'visible',
            Gio.SettingsBindFlags.GET);

        this._fullscreenChangedId =
            global.display.connect('in-fullscreen-changed',
                () => this._updateMonitorConstraint());
        this._updateMonitorConstraint();

        this._settings.connect('changed::color',
            () => this._updateStyles());
        this._settings.connect('changed::background-color',
            () => this._updateStyles());
        this._updateStyles();
    }

    _getColorSetting(key) {
        const str = this._settings.get_string(key);
        const [valid, color] = Clutter.Color.from_string(str);
        if (!valid)
            return '';
        const { red, green, blue, alpha } = color;
        return `${key}: rgba(${red},${green},${blue},${alpha / 255});`;
    }

    _updateMonitorConstraint() {
        const {index} = this._monitorConstraint;
        this._monitorConstraint.work_area =
            !global.display.get_monitor_in_fullscreen(index);
    }

    _updateStyles() {
        const bgStyle = this._getColorSetting('background-color');
        const fgStyle = this._getColorSetting('color');
        const style = `${bgStyle}${fgStyle}`;
        this._topBanner.set({ style });
        this._bottomBanner.set({ style });
    }
});

class Extension {
    constructor() {
        this._banners = [];
    }

    _updateMonitors() {
        const { monitors, panelBox, primaryIndex } = Main.layoutManager;
        if (monitors.length !== this._banners.length) {
            this._clearBanners();

            for (let i = 0; i < monitors.length; i++) {
                const banner = new ClassificationBanner(i);
                Main.uiGroup.add_child(banner);
                this._banners.push(banner);
            }
        }

        const primaryBanner = this._banners[primaryIndex];
        if (primaryBanner)
            Main.uiGroup.set_child_below_sibling(primaryBanner, panelBox);
    }

    _clearBanners() {
        this._banners.forEach(b => b.destroy());
        this._banners = [];
    }

    enable() {
        this._monitorsChangedId = Main.layoutManager.connect('monitors-changed',
            () => this._updateMonitors());
        this._updateMonitors();
    }

    disable() {
        if (this._monitorsChangedId) {
            Main.layoutManager.disconnect(this._monitorsChangedId);
            delete this._monitorsChangedId;
        }
        this._clearBanners();
    }
}

/**
 * @returns {Extension} - the extension's state object
 */
function init() {
    return new Extension();
}
