/* exported init buildPrefsWidget */
const { Gdk, Gio, GObject, Gtk } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Gettext = imports.gettext.domain('gnome-shell-extensions');
const _ = Gettext.gettext;

const Me = ExtensionUtils.getCurrentExtension();
const Adw = Me.imports.adwShim;

let settings = null;

const GenericPrefs = GObject.registerClass(
class GenericPrefs extends Adw.PreferencesGroup {
    _init() {
        super._init();

        this._actionGroup = new Gio.SimpleActionGroup();
        this.insert_action_group('options', this._actionGroup);

        this._actionGroup.add_action(settings.create_action('top-banner'));
        this._actionGroup.add_action(settings.create_action('bottom-banner'));
        this._actionGroup.add_action(settings.create_action('system-info'));

        this.add(this._createSettingsRow('Top banner', 'top-banner'));
        this.add(this._createSettingsRow('Bottom banner', 'bottom-banner'));
        this.add(this._createSettingsRow('System info', 'system-info'));
    }

    _createSettingsRow(title, key) {
        const activatableWidget = new Gtk.Switch({
            valign: Gtk.Align.CENTER,
            action_name: `options.${key}`,
        });
        const row = new Adw.ActionRow({
            activatableWidget,
            title,
        });
        row.add_prefix(activatableWidget);

        return row;
    }
});

const BannerPreset = GObject.registerClass({
    Properties: {
        'message': GObject.ParamSpec.string(
            'message', 'message', 'message',
            GObject.ParamFlags.READWRITE,
            null),
        'color': GObject.ParamSpec.string(
            'color', 'color', 'color',
            GObject.ParamFlags.READWRITE,
            null),
        'background-color': GObject.ParamSpec.string(
            'background-color', 'background-color', 'background-color',
            GObject.ParamFlags.READWRITE,
            null),
    },
}, class BannerPreset extends GObject.Object {});

const AppearancePrefs = GObject.registerClass(
class AppearancePrefs extends Adw.PreferencesGroup {
    _init() {
        super._init();

        const model = new Gio.ListStore({ item_type: BannerPreset.$gtype });
        model.append(new BannerPreset({
            message: 'UNCLASSIFIED',
            color: '#fff',
            background_color: 'rgba(0, 122, 51, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'CONFIDENTIAL',
            color: '#fff',
            background_color: 'rgba(0, 51, 160, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'SECRET',
            color: '#fff',
            background_color: 'rgba(200, 16, 46, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'TOP SECRET',
            color: '#fff',
            background_color: 'rgba(255, 103, 31, 0.75)',
        }));
        model.append(new BannerPreset({
            message: 'TOP SECRET//SCI',
            color: '#000',
            background_color: 'rgba(247, 234, 72, 0.75)',
        }));

        let row, activatableWidget;
        row = this._createPresetsRow(model);
        row.connect('notify::selected-item', comboRow => {
            const { message, color, backgroundColor } = comboRow.selected_item;
            settings.set_string('message', message);
            settings.set_string('color', color);
            settings.set_string('background-color', backgroundColor);
        });
        this.add(row);

        activatableWidget = new Gtk.Entry({
            valign: Gtk.Align.CENTER,
        });
        settings.bind('message',
            activatableWidget, 'text',
            Gio.SettingsBindFlags.DEFAULT);
        row = new Adw.ActionRow({ title: _('Message'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this._createColorButton('background-color', {
            use_alpha: true,
        });
        row = new Adw.ActionRow({ title: _('Background color'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this._createColorButton('color');
        row = new Adw.ActionRow({ title: _('Text color'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);
    }

    _createPresetsRow(model) {
        const listFactory = new Gtk.SignalListItemFactory();
        listFactory.connect('setup',
            (f, item) => item.set_child(new Gtk.Label()));
        listFactory.connect('bind', (f, listItem) => {
            const { child, item } = listItem;

            const provider = new Gtk.CssProvider();
            provider.load_from_data(`* {
                border-radius: 99px;
                padding: 6px;
                color: ${item.color};
                background-color: ${item.background_color};
            }`, -1);
            child.get_style_context().add_provider(provider,
                Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
            child.label = item.message;
        });

        return new Adw.ComboRow({
            title: _('Presets'),
            model,
            listFactory,
            expression: Gtk.ConstantExpression.new_for_value(''),
        });
    }

    _createColorButton(key, params = {}) {
        const rgba = new Gdk.RGBA();
        rgba.parse(settings.get_string(key));

        const button = new Gtk.ColorButton({
            ...params,
            rgba,
            valign: Gtk.Align.CENTER,
        });
        settings.connect(`changed::${key}`, () => {
            const newRgba = new Gdk.RGBA();
            newRgba.parse(settings.get_string(key));
            if (!newRgba.equal(button.rgba))
                button.set({ rgba: newRgba });
        });
        button.connect('notify::rgba',
            () => settings.set_string(key, button.rgba.to_string()));
        return button;
    }
});

const ClassificationPrefs = GObject.registerClass(
class ClassificationPrefs extends Adw.PreferencesPage {
    _init() {
        super._init();

        this.add(new AppearancePrefs());
        this.add(new GenericPrefs());
    }
});

/** */
function init() {
    Adw.init();
    ExtensionUtils.initTranslations();
    settings = ExtensionUtils.getSettings();
}

/**
 * @returns {Gtk.Widget} - the prefs widget
 */
function buildPrefsWidget() {
    return new ClassificationPrefs();
}
